"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const moment = require("moment");
const CacheObject_1 = require("../types/CacheObject");
class CacheHelper {
    static ToCache(name, data, cacheExpiresAfter) {
        try {
            localStorage.setItem(name.toLowerCase(), JSON.stringify(new CacheObject_1.CacheObject({ data, expires: moment().add(cacheExpiresAfter, "m").toDate() })));
        }
        catch (ex) {
            if (ex.name === "QuotaExceededError" || ex.name === "DOMException") {
                const currentUser = localStorage.getItem("currentUser");
                const accessRules = localStorage.getItem("accessRules");
                localStorage.clear();
                if (currentUser) {
                    localStorage.setItem("currentUser", currentUser);
                }
                if (accessRules) {
                    localStorage.setItem("accessRules", accessRules);
                }
                try {
                    localStorage.setItem(name.toLowerCase(), JSON.stringify(new CacheObject_1.CacheObject({ data, expires: moment().add(cacheExpiresAfter, "m").toDate() })));
                }
                catch (exception) {
                    if (exception.name === "QuotaExceededError") {
                    }
                    else {
                        throw exception;
                    }
                }
            }
            else {
                throw ex;
            }
        }
    }
    static TryFromCache(fetchPromise, cacheExpiresAfter, method, url, data) {
        const cacheName = this.FormCacheName(url, data);
        return new Promise((resolve) => {
            const storage = localStorage.getItem(cacheName);
            if (storage) {
                const cache = JSON.parse(storage);
                if (moment(cache.expires).isAfter() && cache.data.length > 0) {
                    resolve(cache.data);
                }
                else {
                    fetchPromise().then((response) => {
                        CacheHelper.ToCache(cacheName, response, cacheExpiresAfter);
                        resolve(response);
                    });
                }
            }
            else {
                fetchPromise().then((response) => {
                    CacheHelper.ToCache(cacheName, response, cacheExpiresAfter);
                    resolve(response);
                });
            }
        });
    }
    static TryOperationPromiseFromCache(fetchPromise, cacheExpiresAfter, method, url, data) {
        const cacheName = this.FormCacheName(url, data);
        return new Promise((resolve) => {
            const storage = localStorage.getItem(cacheName);
            if (storage) {
                const cache = JSON.parse(storage);
                if (moment(cache.expires).isAfter() && cache.data.length > 0) {
                    resolve(cache.data);
                }
                else {
                    fetchPromise().then((response) => {
                        CacheHelper.ToCache(cacheName, response.result, cacheExpiresAfter);
                        resolve(response.result);
                    });
                }
            }
            else {
                fetchPromise().then((response) => {
                    CacheHelper.ToCache(cacheName, response.result, cacheExpiresAfter);
                    resolve(response.result);
                });
            }
        });
    }
    static TryOperationSearchResponseFromCache(fetchPromise, cacheExpiresAfter, method, url, data) {
        const cacheName = this.FormCacheName(url, data);
        return new Promise((resolve) => {
            const storage = localStorage.getItem(cacheName);
            if (storage) {
                const cache = JSON.parse(storage);
                if (moment(cache.expires).isAfter() && cache.data.length > 0) {
                    resolve(cache.data);
                }
                else {
                    fetchPromise().then((response) => {
                        CacheHelper.ToCache(cacheName, response.result.data, cacheExpiresAfter);
                        resolve(response.result.data);
                    });
                }
            }
            else {
                fetchPromise().then((response) => {
                    CacheHelper.ToCache(cacheName, response.result.data, cacheExpiresAfter);
                    resolve(response.result.data);
                });
            }
        });
    }
    static FormCacheName(url, data) {
        let res = "";
        if (data) {
            res = `${url}_${this.StringifyData(data)}`;
        }
        else {
            res = url;
        }
        res = res.toLowerCase();
        return res;
    }
    static StringifyData(data) {
        const replaceArr = [/\{/, /\}/, /\"/, /\:/, /\,/, /\[/, /\]/];
        let str = JSON.stringify(data);
        replaceArr.forEach(x => {
            str = str.replace(new RegExp(x, "g"), "");
        });
        return str;
    }
}
exports.CacheHelper = CacheHelper;
