"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const ayax_common_helpers_1 = require("ayax-common-helpers");
const ayax_common_types_1 = require("ayax-common-types");
const CacheHelper_1 = require("../helpers/CacheHelper");
class CacheService {
    constructor(operationService, cacheExpiresAfter, staticCache) {
        this._operationService = operationService;
        this._cacheExpiresAfter = cacheExpiresAfter ? cacheExpiresAfter : 15;
        this._staticCache = staticCache ? staticCache : {};
    }
    Get(url) {
        return CacheHelper_1.CacheHelper.TryFromCache(() => this.Fetch("get", url), this._cacheExpiresAfter, "get", url, null);
    }
    Post(url, data) {
        return CacheHelper_1.CacheHelper.TryFromCache(() => this.Fetch("post", url, data), this._cacheExpiresAfter, "post", url, data);
    }
    List(dictionary, method, disableSort) {
        return __awaiter(this, void 0, void 0, function* () {
            const url = method ? `/${dictionary}/${method}` : `/${dictionary}/list`;
            let result = yield CacheHelper_1.CacheHelper.TryFromCache(() => this.Fetch("get", url), this._cacheExpiresAfter, "get", url, null);
            if (disableSort) {
                return result;
            }
            return result.sort(ayax_common_helpers_1.ArraySortHelper.sortBy(["order", "title"]).asc);
        });
    }
    ListAsDictionary(dictionary, method, disableSort) {
        return __awaiter(this, void 0, void 0, function* () {
            const cacheDictionary = {};
            (yield this.List(dictionary, method, disableSort)).forEach(x => {
                cacheDictionary[x.id] = x;
            });
            return cacheDictionary;
        });
    }
    ListAsSelectItems(dictionary, method, disableSort) {
        return __awaiter(this, void 0, void 0, function* () {
            return (yield this.List(dictionary, method, disableSort)).map(x => new ayax_common_types_1.SelectItem({ text: x.name ? x.name : x.title, value: x.id, disabled: x.isDisabled }));
        });
    }
    Search(dictionary, data, method) {
        const url = method ? `/${dictionary}/${method}` : `/${dictionary}/search`;
        return CacheHelper_1.CacheHelper.TryFromCache(() => this.Fetch("search", url, data), this._cacheExpiresAfter, "search", url, data);
    }
    Fetch(method, url, data) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                if (this._staticCache[url]) {
                    return this._staticCache[url];
                }
                switch (method.toLocaleLowerCase()) {
                    case "post":
                        return (yield this._operationService.post(url, data)).ensureSuccess();
                    case "search":
                        return (yield this._operationService.post(url, data)).ensureSuccess().data;
                    default:
                        return (yield this._operationService.get(url)).ensureSuccess();
                }
            }
            catch (e) {
                console.error(`Ошибка получения справочника url=${url} method=${method} data=${JSON.stringify(data)} ${JSON.stringify(e)}`);
            }
            return [];
        });
    }
}
exports.CacheService = CacheService;
